@@ -1,4 +1,9 @@
 package Data::Schema;
+our $VERSION = '0.12';
+
+
+# ABSTRACT: Validate nested data structures with nested structure
+
 
 use Moose;
 use vars qw(@ISA @EXPORT);
@@ -8,73 +13,18 @@ require Exporter;
 use Data::Schema::Config;
 use Data::PrefixMerge;
 use Data::Schema::Type::Schema;
+use Digest::MD5 qw/md5_hex/;
+use Storable qw/freeze/;
 
-=head1 NAME
+# don't blame me if you/someone else changes this
+$Storable::canonical = 1;
+$Storable::forgive_me = 1; # config can contain coderefs, e.g in gettext_function
 
-Data::Schema - Validate nested data structures with nested structure
-
-=head1 VERSION
-
-Version 0.11
-
-=cut
-
-our $VERSION = '0.11';
-
-=head1 SYNOPSIS
-
-    # OO interface
-    use Data::Schema;
-    my $validator = Data::Schema->new();
-    my $schema = [array => {min_len=>2, max_len=>4}];
-    my $data = [1, 2, 3];
-    my $res = $validator->validate($data, $schema);
-    print "valid!" if $res->{success}; # prints 'valid!'
-
-    # procedural interface
-    use Data::Schema;
-    my $sch = ["hash",
-               {keys =>
-                    {name => "str",
-                     age  => ["int", {required=>1, min=>18}]
-                    }
-                }
-              ];
-    my $r;
-    $r = ds_validate({name=>"Lucy", age=>18}, $sch); # success
-    $r = ds_validate({name=>"Lucy"         }, $sch); # fail: missing age
-    $r = ds_validate({name=>"Lucy", age=>16}, $sch); # fail: underage
-
-=head1 DESCRIPTION
-
-Data::Schema (DS) is a schema system for data validation. It lets you write
-schemas as data structures, ranging from very simple (a scalar) to complex
-(nested hashes/arrays) depending on how complex you want your validation to be.
-
-Writing schemas as data structures themselves has several advantages. First, it
-is more portable across languages (e.g. using YAML to share schemas between
-Perl, Python, PHP, Ruby). Second, you can validate the schema using the schema
-system itself. Third, it is easy to generate code, help message (e.g. so-called
-"usage" for function/command line script), etc. from the schema.
-
-Potential application of DS: validating configuration, function
-parameters, command line arguments, etc.
-
-To get started, see L<Data::Schema::Manual::Tutorial>.
-
-
-=head1 FUNCTIONS
-
-=head2 ds_validate($data, $schema)
-
-Non-OO wrapper for validate(). Exported by default. See C<validate()> method.
-
-=cut
 
 sub ds_validate {
     my ($data, $schema) = @_;
-    my $validator = __PACKAGE__->new(schema => $schema);
-    $validator->validate($data);
+    my $ds = __PACKAGE__->new(schema => $schema);
+    $ds->validate($data);
 }
 
 my $Merger = new Data::PrefixMerge;
@@ -82,18 +32,11 @@ $Merger->config->recurse_array(1);
 $Merger->config->preserve_keep_prefix(1);
 
 
-=head1 ATTRIBUTES
-
-=cut
 
 has plugins => (is => 'rw');
 has type_handlers => (is => 'rw');
+has compiled_subnames => (is => 'rw'); # for lexical visibility
 
-=head2 config
-
-Configuration object. See L<Data::Schema::Config>.
-
-=cut
 
 has config => (is => 'rw');
 
@@ -103,16 +46,17 @@ has validation_state_stack => (is => 'rw');
 has schema => (is => 'rw');
 # has data_copy?
 has too_many_errors => (is => 'rw');
-has too_many_warnings => (is => 'rw');
 has errors => (is => 'rw');
-has warnings => (is => 'rw');
 has data_pos => (is => 'rw');
 has schema_pos => (is => 'rw');
+has stash => (is => 'rw'); # for storing stuffs during validation
 
+has compilation_state_stack => (is => 'rw');
 
-=head1 METHODS
+# compilation state
+# stash
+has outer_stash => (is => 'rw'); # same as stash, but won't be reset during inner calls to _emit_perl
 
-=cut
 
 sub BUILD {
     my ($self, $args) = @_;
@@ -122,7 +66,7 @@ sub BUILD {
         # some sanity checks
         my $is_hashref = ref($self->config) eq 'HASH';
         die "config must be a hashref or a Data::Schema::Config" unless
-            $is_hashref || UNIVERSAL::isa("Data::Schema::Config");
+            $is_hashref || UNIVERSAL::isa($self->config, "Data::Schema::Config");
         $self->config(Data::Schema::Config->new(%{ $self->config })) if $is_hashref;
         die "config->schema_search_path must be an arrayref" unless ref($self->config->schema_search_path) eq 'ARRAY';
     } else {
@@ -170,96 +114,66 @@ sub BUILD {
         $self->register_plugin("Data::Schema::Plugin::$_") for @defpl;
     }
 
-    $self->validation_state_stack([]) unless $self->validation_state_stack;
+    $self->validation_state_stack([])  unless $self->validation_state_stack;
+    $self->compilation_state_stack([]) unless $self->compilation_state_stack;
+    $self->compiled_subnames({})       unless $self->compiled_subnames;
 };
 
-=head2 merge_attr_hashes($attr_hashes)
-
-Merge several attribute hashes if there are hashes that can be merged (i.e.
-contains merge prefix in its keys). Used by DST::Base and DST::Schema. As DS
-user, normally you wouldn't need this.
-
-=cut
 
 sub merge_attr_hashes {
     my ($self, $attr_hashes) = @_;
     my @merged;
     my $res = {error=>''};
 
-    my $i = 0;
-    while (1) {
-        last if $i >= @$attr_hashes;
-        my $attr_hash = $attr_hashes->[$i];
-        my $has_merge_prefix = grep {/^[*+.!^-]/} keys %$attr_hash;
-        if ($i > 0 && $has_merge_prefix) {
-            my $mres = $Merger->merge($attr_hashes->[$i-1], $attr_hash);
-            if (!$mres->{success}) {
-                $res->{error} = $mres->{error};
-                last;
-            }
-            $merged[-1] = $mres->{result};
-        } else {
-            push @merged, $attr_hashes->[$i];
+    my $i = -1;
+    while (++$i < @$attr_hashes) {
+        if (!$i) { push @merged, $attr_hashes->[$i]; next }
+        my $has_merge_prefix = grep {/^[*+.!^-]/} keys %{ $attr_hashes->[$i] };
+        if (!$has_merge_prefix) { push @merged, $attr_hashes->[$i]; next }
+        my $mres = $Merger->merge($merged[-1], $attr_hashes->[$i]);
+        #print "DEBUG: prefix_merge $i (".Data::Schema::Type::Base::_dump({}, $merged[-1]).", ".
+        #    Data::Schema::Type::Base::_dump({}, $attr_hashes->[$i])." = ".($mres->{success} ? Data::Schema::Type::Base::_dump({}, $mres->{result}) : "FAIL")."\n";
+        if (!$mres->{success}) {
+            $res->{error} = $mres->{error};
+            last;
         }
-        $i++;
+        $merged[-1] = $mres->{result};
     }
     $res->{result} = \@merged unless $res->{error};
     $res->{success} = !$res->{error};
+
+    $Merger->remove_keep_prefixes(\@merged);
+
+    #print "DEBUG: merge_attr_hashes($self, ".Data::Schema::Type::Base::_dump({}, $attr_hashes).
+    #    ") = ".($res->{success} ? Data::Schema::Type::Base::_dump({}, $res->{result}) : "FAIL")."\n";
     $res;
 }
 
-=head2 init_validation_state()
-
-Initialize validation state. Used internally by validate(). As DS user, normally
-you wouldn't need this.
-
-=cut
 
 sub init_validation_state {
     my ($self) = @_;
     $self->schema(undef);
     $self->errors([]);
-    $self->warnings([]);
     $self->too_many_errors(0);
-    $self->too_many_warnings(0);
     $self->data_pos([]);
     $self->schema_pos([]);
+    $self->stash({});
 }
 
-=head2 save_validation_state()
-
-Save validation state (position in data, position in schema, number of errors,
-etc) into a stack, so that you can start using the validator to validate a new
-data with a new schema, even in the middle of validating another data/schema.
-Used internally by validate() and DST::Schema. As DS user, normally you wouldn't
-need this.
-
-See also: B<restore_validation_state()>.
-
-=cut
 
 sub save_validation_state {
     my ($self) = @_;
     my $state = {
         schema => $self->schema,
         errors => $self->errors,
-        warnings => $self->warnings,
         too_many_errors => $self->too_many_errors,
-        too_many_warnings => $self->too_many_warnings,
         data_pos => $self->data_pos,
         schema_pos => $self->schema_pos,
+        stash => $self->stash,
     };
     push @{ $self->validation_state_stack }, $state;
 }
 
-=head2 restore_validation_state()
-
-Restore the last validation state into a stack. Used internally by validate()
-and DST::Schema. As DS user, normally you wouldn't need this.
-
-See also: B<save_validation_state()>.
-
-=cut
 
 sub restore_validation_state {
     my ($self) = @_;
@@ -267,16 +181,39 @@ sub restore_validation_state {
     die "Can't restore validation state, stack is empty!" unless $state;
     $self->schema($state->{schema});
     $self->errors($state->{errors});
-    $self->warnings($state->{warnings});
     $self->too_many_errors($state->{too_many_errors});
-    $self->too_many_warnings($state->{too_many_warnings});
     $self->data_pos($state->{data_pos});
     $self->schema_pos($state->{schema_pos});
+    $self->stash($state->{stash});
+}
+
+
+sub init_compilation_state {
+    my ($self, $inner) = @_;
+    $self->stash({});
+    $self->schema_pos([]) unless $self->schema_pos;
+    $self->outer_stash({compiling=>1}) unless $inner;
+}
+
+
+sub save_compilation_state {
+    my ($self) = @_;
+    my $state = {
+        stash => $self->stash,
+    };
+    push @{ $self->compilation_state_stack }, $state;
+}
+
+
+sub restore_compilation_state {
+    my ($self) = @_;
+    my $state = pop @{ $self->compilation_state_stack };
+    die "Can't restore validation state, stack is empty!" unless $state;
+    $self->stash($state->{stash});
 }
 
 sub _log {
     my ($self, $stack, $limit, $message, $too_many_msgs) = @_;
-    #print "log: ".$self->_pos_as_str($self->data_pos).": $message\n";
     if (defined($limit) && $limit > 0) {
         if (@$stack >= $limit) {
             push @$stack, [[], [], $too_many_msgs];
@@ -286,32 +223,42 @@ sub _log {
     push @$stack, [[ @{$self->data_pos} ], [ @{$self->schema_pos} ], $message];
 }
 
-=head2 log_error($message)
+sub emitpl_my {
+    my ($self, @varnames) = @_;
+    join("", map { !$self->stash->{"C_var_$_"}++ ? "my $_;\n" : "" } @varnames);
+}
 
-Add an error when in validation process. Will not add if there are already too
-many errors (C<too_many_errors> attribute is true). Used by type handlers. As DS
-user, normally you wouldn't need this.
+sub emitpl_require {
+    my ($self, @modnames) = @_;
+    join("", map { !$self->stash->{"C_req_$_"}++ ? "require $_;\n" : "" } @modnames);
+}
 
-=cut
 
-sub log_error {
+sub data_error {
     my ($self, $message) = @_;
     $self->too_many_errors(1) unless
         $self->_log($self->errors, $self->config->max_errors, $message, "too many errors");
 }
 
-=head2 log_warning($message)
+sub emitpl_data_error {
+    my ($self, $msg, $is_literal) = @_;
 
-Add a warning when in validation process. Will not add if there are already too
-many warnings (C<too_many_warnings> attribute is true). Used by type handlers.
-As DS user, normally you wouldn't need this.
+    my $lit;
+    if ($is_literal) {
+        $lit = $msg;
+    } else {
+        $msg =~ s/(['\\])/\\$1/g;
+        $lit = "'$msg'";
+    }
+    # we don't track schema pos because everything is assumed to be
+    # compiled to a black-box perl sub
+    'push @errors, [[@$datapos],[@$schemapos],'.$lit.']; if (@errors >= '.$self->config->max_errors.") { last L1 }";
+}
 
-=cut
 
-sub log_warning {
+sub schema_error {
     my ($self, $message) = @_;
-    $self->too_many_warnings(1) unless
-        $self->_log($self->warnings, $self->config->max_warnings, $message, "too many warnings");
+    die "Schema error during validation/compilation: $message";
 }
 
 sub _pos_as_str {
@@ -321,17 +268,6 @@ sub _pos_as_str {
     $res;
 }
 
-=head2 check_type_name($name)
-
-Checks whether C<$name> is a valid type name. Returns true if valid,
-false if invalid. By default it requires that type name starts with a
-lowercase letter and contains only lowercase letters, numbers, and
-underscores. Maximum length is 64.
-
-You can override this method if you want stricter/looser type name
-criteria.
-
-=cut
 
 sub check_type_name {
     my ($self, $name) = @_;
@@ -351,34 +287,6 @@ sub _load_type_handler {
     $obj;
 }
 
-=head2 register_type($name, $class|$obj)
-
-Register a new type, along with a class name (C<$class>) or the actual object
-(C<$obj>) to handle the type. If C<$class> is given, the class will be require'd
-and instantiated to become object later when needed via get_type_handler.
-
-Any object can become a type handler, as long as it has:
-
-=over
-
-=item *
-
-a C<validator()> rw property to store/set validator object;
-
-=item *
-
-C<handle_type()> method to handle type checking;
-
-=item *
-
-zero or more C<handle_attr_*()> methods to handle attribute checking.
-
-=back
-
-See L<Data::Schema::Manual::TypeHandler> for more details on writing a type
-handler.
-
-=cut
 
 sub register_type {
     my ($self, $name, $obj_or_class) = @_;
@@ -398,30 +306,6 @@ sub register_type {
     }
 }
 
-=head2 register_plugin($class|$obj)
-
-Register a new plugin. Accept a plugin object or class. If C<$class> is given,
-the class will be require'd (if not already require'd) and instantiated to
-become object.
-
-Any object can become a plugin, you don't need to subclass from anything, as
-long as it has:
-
-=over 4
-
-=item *
-
-a C<validator()> rw property to store/set validator object;
-
-=item *
-
-zero or more C<handle_*()> methods to handle some events/hooks.
-
-=back
-
-See L<Data::Schema::Manual::Plugin> for more details on writing a plugin.
-
-=cut
 
 sub register_plugin {
     my ($self, $obj_or_class) = @_;
@@ -438,38 +322,27 @@ sub register_plugin {
     push @{ $self->plugins }, $obj;
 }
 
-=head2 call_handler($name, [@args])
-
-Try handle_*() method from each registered plugin until one returns 0 or 1. If
-a plugin return -1 (decline) then we continue to the next plugin. Returns the
-status of the last plugin. Returns -1 if there's no handler to invoke.
-
-=cut
 
 sub call_handler {
     my ($self, $name, @args) = @_;
     $name = "handle_$name" unless $name =~ /^handle_/;
     for my $p (@{ $self->plugins }) {
         if ($p->can($name)) {
-            my $res = $p->$name(@args);
+            #print "DEBUG: calling plugin $p, handler $name ...\n";
+	    my $res = $p->$name(@args);
+            #print "DEBUG: res = $res ...\n";
             return $res if $res != -1;
         }
     }
     -1;
 }
 
-=head2 get_type_handler($name)
-
-Try to get type handler for a certain type. If type handler is not an object (a
-class name), instantiate it first. If type is not found, invoke
-handle_unknown_type() in plugins to give plugins a chance to load the type. If
-type is still not found, return undef.
-
-=cut
 
 sub get_type_handler {
     my ($self, $name) = @_;
     my $th;
+    #print "DEBUG: Getting type handler for type $name ...\n";
+    #use Data::Dumper; print "DEBUG: Current type handlers: ", Data::Dumper->new([$self->type_handlers])->Indent(1)->Dump;
     if (!($th = $self->type_handlers->{$name})) {
         # let's give plugin a chance to do something about it and then try again
         if ($self->call_handler("unknown_type", $name) > 0) {
@@ -480,16 +353,10 @@ sub get_type_handler {
             $th = $self->_load_type_handler($name);
         }
     }
+    #use Data::Dumper; print "DEBUG: Type handler got: ".Dumper($th)."\n";
     $th;
 }
 
-=head2 normalize_schema($schema)
-
-Normalize a schema into the third form (hash form) ({type=>...,
-attr_hashes=>..., def=>...) as well as do some sanity checks on it. Returns an
-error message string if fails.
-
-=cut
 
 sub normalize_schema {
     my ($self, $schema) = @_;
@@ -561,14 +428,6 @@ sub normalize_schema {
     return "schema must be a str, arrayref, or hashref";
 }
 
-=head2 register_schema_as_type($schema, $name)
-
-Register schema as new type. $schema is a normalized schema. Return {success=>(0
-or 1), error=>...}. Fails if type with name B<$name> is already defined, or if
-$schema cannot be parsed. Might actually register more than one type actually,
-if the schema contains other types in it (hash form of schema can define types).
-
-=cut
 
 sub register_schema_as_type {
     my ($self, $nschema, $name, $path) = @_;
@@ -593,7 +452,7 @@ sub register_schema_as_type {
                 }
             }
         }
-        my $th = Data::Schema::Type::Schema->new(nschema=>$nschema);
+        my $th = Data::Schema::Type::Schema->new(nschema=>$nschema, name=>$name);
         $self->register_type($name => $th);
         last;
     }
@@ -601,16 +460,6 @@ sub register_schema_as_type {
     $res;
 }
 
-=head2 validate($data[, $schema])
-
-Validate a data structure. $schema must be given unless you already give the
-schema via the B<schema> attribute.
-
-Returns {success=>0 or 1, errors=>[...], warnings=>[...]}. The 'success' key
-will be set to 1 if the data validates, otherwise 'errors' and 'warnings' will
-be filled with the details.
-
-=cut
 
 sub validate {
     my ($self, $data, $schema) = @_;
@@ -618,60 +467,79 @@ sub validate {
     $schema ||= $self->schema;
 
     $self->init_validation_state();
+    $self->init_compilation_state() if $self->config->compile;
     $self->_validate($data, $schema);
     $self->schema($saved_schema);
 
     {success  => !@{$self->errors},
      errors   => [$self->errors_as_array],
-     warnings => [$self->warnings_as_array],
     };
 }
 
-=head2 errors_as_array
-
-Return formatted errors in an array of strings.
-
-=cut
 
 sub errors_as_array {
     my ($self) = @_;
     map { sprintf "data\@%s schema\@%s %s", $self->_pos_as_str($_->[0]), $self->_pos_as_str($_->[1]), $_->[2] } @{ $self->errors };
 }
 
-=head2 warnings_as_array
+sub _schema2csubname {
+    my ($self, $schema) = @_;
+    my $n1 = defined($schema) ? (ref($schema) ? md5_hex(freeze($schema)) : $schema) : "";
+    my $n2 = md5_hex(freeze($self->config));
+    "__cs_${n1}_$n2";
+}
 
-Return formatted warnings in an array of strings.
 
-=cut
+sub emit_perl {
+    my ($self, $schema, $inner) = @_;
+    $self->init_compilation_state($inner);
+    $self->_emit_perl(undef, $schema);
+}
 
-sub warnings_as_array {
-    my ($self) = @_;
-    map { sprintf "data\@%s schema\@%s %s", $self->_pos_as_str($_->[0]), $self->_pos_as_str($_->[1]), $_->[2] } @{ $self->warnings };
+sub _emit_perl {
+    _validate_or_emit_perl(@_, 'EMIT_PERL');
 }
 
 # the difference between validate() and _validate(): _validate() is not for the
 # end-user, it doesn't initialize validation state and so can be used in the
 # middle of another validation (e.g. for validating schema types). _validate()
-# also doesn't format and returns the list of errors/warnings, you need to get
-# them yourself from the validator.
+# also doesn't format and returns the list of errors, you need to get them
+# yourself from the validator.
 
 sub _validate {
     my ($self, $data, $schema) = @_;
+    _validate_or_emit_perl(@_, 'VALIDATE');
+}
+
+sub _validate_or_emit_perl {
+    my ($self, $data, $schema, $action) = @_;
+
+    die "Schema must be specified" unless defined($schema);
+
+    my $compile = $self->config->compile;
+    my $csubname = $self->_schema2csubname($schema);
+    if ($compile && $action eq 'VALIDATE' && $self->compiled_subnames->{$csubname}) {
+	#print "HIT!\n";
+	goto LV1;
+    }
 
     # since schema may define types inside it, we save the original types list
     # so we can register new types and then restore back to original state
     # later.
     my $orig_type_handlers;
+    my $orig_compiled_subnames;
 
-    while (1) {
+    {
         my $s = $self->normalize_schema($schema);
         if (!ref($s)) {
-            $self->log_error("schema error: $s");
+            $self->schema_error($s);
             last;
         }
 
         if ($s->{def}) {
+	    #print "DEBUG: Saving type handlers\n";
             $orig_type_handlers = { %{$self->type_handlers} };
+            $orig_compiled_subnames = { %{$self->compiled_subnames} };
             push @{ $self->schema_pos }, 'def', '';
             my $has_err;
             for (keys %{ $s->{def} }) {
@@ -679,13 +547,13 @@ sub _validate {
                 my $subs = $self->normalize_schema($s->{def}{$_});
                 if (!ref($subs)) {
                     $has_err++;
-                    $self->log_error("normalize schema type error: $s");
+                    $self->data_error("normalize schema type error: $s");
                     last;
                 }
                 my $res = $self->register_schema_as_type($subs, $_);
                 if (!$res->{success}) {
                     $has_err++;
-                    $self->log_error("register schema type error: $res->{error}");
+                    $self->data_error("register schema type error: $res->{error}");
                     last;
                 }
             }
@@ -696,17 +564,403 @@ sub _validate {
 
         my $th = $self->get_type_handler($s->{type});
         if (!$th) {
-            $self->log_error("schema error: unknown type `$s->{type}'");
+            $self->schema_error("unknown type `$s->{type}'");
             last;
         }
-        $th->handle_type($data, $s->{attr_hashes});
-        last;
+        if ($compile || $action eq 'EMIT_PERL') {
+            my $code = $th->emit_perl($s->{attr_hashes}, $csubname);
+            return $code if $action eq 'EMIT_PERL';
+            if (!$code) {
+                $self->schema_error("no Perl code generated");
+                last;
+            }
+	    unless ($Data::Schema::__compiled::{$csubname}) {
+		eval "package Data::Schema::__compiled; $code; package Data::Schema;";
+		my $eval_error = $@;
+		if ($eval_error) {
+		    my $i=1; my @c; for (split /\n/, $code) { push @c, sprintf "%4d|%s\n", $i++, $_ } $code = join "", @c;
+		    print STDERR $code;
+		    print STDERR $eval_error;
+		    die "Can't compile code: $eval_error";
+		}
+	    }
+	    $self->compiled_subnames->{$csubname} = 1;
+        } else {
+            $th->handle_type($data, $s->{attr_hashes});
+        }
     }
 
-    $self->type_handlers($orig_type_handlers) if $orig_type_handlers;
+    if ($orig_type_handlers) {
+	#print "DEBUG: Restoring original type handlers\n";
+	$self->type_handlers($orig_type_handlers);
+	$self->compiled_subnames($orig_compiled_subnames);
+    }
+
+  LV1:
+    # execute compiled code
+    if ($compile) {
+        no strict 'refs';
+	my ($errors) = "Data::Schema::__compiled::$csubname"->($data);
+	push @{ $self->errors }, @$errors;
+    }
 }
 
 
+sub compile {
+    my ($self, $schema) = @_;
+    my $csubname = $self->_schema2csubname($schema);
+    unless ($Data::Schema::__compiled::{$csubname}) {
+	$self->save_compilation_state;
+	my $code = $self->emit_perl($schema);
+	$self->restore_compilation_state;
+	die "Can't generate Perl code for schema" unless $code;
+	eval "package Data::Schema::__compiled; $code; package Data::Schema;";
+	my $eval_error = $@;
+	if ($eval_error) {
+	    my $i=1; my @c; for (split /\n/, $code) { push @c, sprintf "%4d|%s\n", $i++, $_ } $code = join "", @c;
+	    print STDERR $code;
+	    print STDERR $eval_error;
+	    die "Can't compile code: $eval_error";
+	}
+    }
+    my $cfullsubname = "Data::Schema::__compiled::$csubname";
+    (\&$cfullsubname, $csubname);
+}
+
+sub emitpls_sub {
+    my ($self, $schema) = @_;
+
+    my $csubname = $self->_schema2csubname($schema);
+    my $perl = '';
+    
+    unless ($Data::Schema::__compiled::{$csubname} ||
+	    $self->outer_stash->{"C_def_$csubname"}++) {
+	$self->save_compilation_state;
+	$perl = $self->emit_perl($schema, 1);
+	$self->restore_compilation_state;
+	die "Can't generate Perl code for schema" unless $perl;
+	$self->outer_stash->{"C_def_$csubname"}++;
+    }
+    ($perl, $csubname);
+}
+
+
+__PACKAGE__->meta->make_immutable;
+no Moose;
+1;
+
+__END__
+=pod
+
+=head1 NAME
+
+Data::Schema - Validate nested data structures with nested structure
+
+=head1 VERSION
+
+version 0.12
+
+=head1 SYNOPSIS
+
+    # OO interface
+    use Data::Schema;
+    my $ds = Data::Schema->new();
+    my $schema = [array => {min_len=>2, max_len=>4}];
+    my $data = [1, 2, 3];
+    my $res = $ds->validate($data, $schema);
+    print "valid!" if $res->{success}; # prints 'valid!'
+
+    # procedural interface
+    use Data::Schema;
+    my $sch = ["hash",
+               {keys =>
+                    {name => "str",
+                     age  => ["int", {required=>1, min=>18}]
+                    }
+                }
+              ];
+    my $r;
+    $r = ds_validate({name=>"Lucy", age=>18}, $sch); # success
+    $r = ds_validate({name=>"Lucy"         }, $sch); # fail: missing age
+    $r = ds_validate({name=>"Lucy", age=>16}, $sch); # fail: underage
+
+    # some schema examples
+
+    # -- array
+    "array"
+
+    # -- array of ints
+    [array => {of=>"int"}]
+
+    # -- array of positive, even ints
+    [array => {of=>[int => {min=>0, divisible_by=>2}]}]
+
+    # -- 3x3x3 "multi-dim" arrays
+    [array => {len=>3, of=>
+        [array => {len=>3, of=>
+            [array => {len=>3}]}]}]
+
+    # -- HTTP headers, each header can be a string or array of strings
+    [hash => {
+        required => 1,
+        keys_match => '^\w+(-w+)*$',
+        values_of => [either => {of=>[
+            "str",
+            [array=>{of=>"str", minlen=>1}],
+        ]}],
+    }]
+
+    # -- records (demonstrates subschema and attribute merging). Note:
+    # I am not sexist or anything, just that for the love of g*d I
+    # can't think of a better example atm. it's late...
+    {def => {
+        person => [hash => {
+            keys => {
+                name       => "str",
+                race       => "str",
+                age        => [int => {min=>0, max=>100}],
+            },
+        }],
+
+        # women are like people, but they have additional keys
+        # 'husband' and 'cup_size' (additive) and different age
+        # restriction (replace).
+
+        woman => [person => {
+            '*keys' => {
+                husband    => "str",
+                cup_size   => [str => {one_of=>[qw/AA A B C D DD/]}],
+                '*age'     => [int => {min=>0, max=>120}],
+            },
+        }],
+
+        # girls are like women, but they do not have husbands yet
+        # (remove keys)
+
+        girl => [woman => {
+            '*keys' => {
+                '!husband' => undef,
+            }
+        }],
+
+        girls  => [array => {of=>"girl"}],
+    },
+    type => "girls",
+    };
+
+=head1 DESCRIPTION
+
+Data::Schema (DS) is a schema system for data validation. It lets you
+write schemas as data structures, ranging from very simple (a scalar)
+to fairly complex (nested hashes/arrays with various criteria).
+
+Writing schemas as data structures themselves has several advantages. First, it
+is more portable across languages (e.g. using YAML to share schemas between
+Perl, Python, PHP, Ruby). Second, you can validate the schema using the schema
+system itself. Third, it is easy to generate code, help message (e.g. so-called
+"usage" for function/command line script), etc. from the schema.
+
+Potential application of DS: validating configuration, function
+parameters, command line arguments, etc.
+
+To get started, see L<Data::Schema::Manual::Tutorial>.
+
+=head1 FUNCTIONS
+
+=head2 ds_validate($data, $schema)
+
+Non-OO wrapper for validate(). Exported by default. See C<validate()> method.
+
+=head1 ATTRIBUTES
+
+=head2 config
+
+Configuration object. See L<Data::Schema::Config>.
+
+=head1 METHODS
+
+=head2 merge_attr_hashes($attr_hashes)
+
+Merge several attribute hashes if there are hashes that can be merged (i.e.
+contains merge prefix in its keys). Used by DST::Base and DST::Schema. As DS
+user, normally you wouldn't need this.
+
+=head2 init_validation_state()
+
+Initialize validation state. Used internally by validate(). As DS user, normally
+you wouldn't need this.
+
+=head2 save_validation_state()
+
+Save validation state (position in data, position in schema, number of errors,
+etc) into a stack, so that you can start using the validator to validate a new
+data with a new schema, even in the middle of validating another data/schema.
+Used internally by validate() and DST::Schema. As DS user, normally you wouldn't
+need this.
+
+See also: B<restore_validation_state()>.
+
+=head2 restore_validation_state()
+
+Restore the last validation state from the stack. Used internally by
+validate() and DST::Schema. As DS user, normally you wouldn't need
+this.
+
+See also: B<save_validation_state()>.
+
+=head2 init_compilation_state()
+
+Initialize compilation state. Used internally by emit_perl(). As DS
+user, normally you wouldn't need this.
+
+=head2 save_compilation_state()
+
+Save compilation state. Used internally by emit_perl() and
+DST::Schema. As DS user, normally you wouldn't need this.
+
+See also: B<restore_compilation_state()>.
+
+=head2 restore_compilation_state()
+
+Restore the last compilation state from the stack. Used internally by
+emit_perl() and DST::Schema. As DS user, normally you wouldn't need
+this.
+
+See also: B<save_compilation_state()>.
+
+=head2 data_error($message)
+
+Add a data error when in validation process. Will not add if there are
+already too many errors (C<too_many_errors> attribute is true). Used
+by type handlers. As DS user, normally you wouldn't need this.
+
+=head2 schema_error($message)
+
+Method to call when encountering schema error during
+validation/compilation. Used by type handlers. As DS user, normally
+you wouldn't need this.
+
+=head2 check_type_name($name)
+
+Checks whether C<$name> is a valid type name. Returns true if valid,
+false if invalid. By default it requires that type name starts with a
+lowercase letter and contains only lowercase letters, numbers, and
+underscores. Maximum length is 64.
+
+You can override this method if you want stricter/looser type name
+criteria.
+
+=head2 register_type($name, $class|$obj)
+
+Register a new type, along with a class name (C<$class>) or the actual object
+(C<$obj>) to handle the type. If C<$class> is given, the class will be require'd
+and instantiated to become object later when needed via get_type_handler.
+
+Any object can become a type handler, as long as it has:
+
+=over
+
+=item *
+
+a C<validator()> rw property to store/set validator object;
+
+=item *
+
+C<handle_type()> method to handle type checking;
+
+=item *
+
+zero or more C<handle_attr_*()> methods to handle attribute checking.
+
+=back
+
+See L<Data::Schema::Manual::TypeHandler> for more details on writing a type
+handler.
+
+=head2 register_plugin($class|$obj)
+
+Register a new plugin. Accept a plugin object or class. If C<$class> is given,
+the class will be require'd (if not already require'd) and instantiated to
+become object.
+
+Any object can become a plugin, you don't need to subclass from anything, as
+long as it has:
+
+=over 4
+
+=item *
+
+a C<validator()> rw property to store/set validator object;
+
+=item *
+
+zero or more C<handle_*()> methods to handle some events/hooks.
+
+=back
+
+See L<Data::Schema::Manual::Plugin> for more details on writing a plugin.
+
+=head2 call_handler($name, [@args])
+
+Try handle_*() method from each registered plugin until one returns 0 or 1. If
+a plugin return -1 (decline) then we continue to the next plugin. Returns the
+status of the last plugin. Returns -1 if there's no handler to invoke.
+
+=head2 get_type_handler($name)
+
+Try to get type handler for a certain type. If type handler is not an object (a
+class name), instantiate it first. If type is not found, invoke
+handle_unknown_type() in plugins to give plugins a chance to load the type. If
+type is still not found, return undef.
+
+=head2 normalize_schema($schema)
+
+Normalize a schema into the third form (hash form) ({type=>...,
+attr_hashes=>..., def=>...) as well as do some sanity checks on it. Returns an
+error message string if fails.
+
+=head2 register_schema_as_type($schema, $name)
+
+Register schema as new type. $schema is a normalized schema. Return {success=>(0
+or 1), error=>...}. Fails if type with name B<$name> is already defined, or if
+$schema cannot be parsed. Might actually register more than one type actually,
+if the schema contains other types in it (hash form of schema can define types).
+
+=head2 validate($data[, $schema])
+
+Validate a data structure. $schema must be given unless you already give the
+schema via the B<schema> attribute.
+
+Returns {success=>0 or 1, errors=>[...]}. The 'success' key will be
+set to 1 if the data validates, otherwise 'errors' will be filled with
+the details.
+
+=head2 errors_as_array
+
+Return formatted errors in an array of strings.
+
+=head2 emit_perl([$schema])
+
+Return Perl code equivalent to schema C<$schema>.
+
+If you want to get the compiled code (as a coderef) directly, use
+C<compile>.
+
+=head2 compile($schema)
+
+Compile the schema into Perl code and return a 2-element list:
+($coderef, $subname). $coderef is the resulting subroutine and
+$subname is the subroutine name in the compilation namespace
+(Data::Schema::__compiled).
+
+If the same schema is already compiled, the existing compiled
+subroutine is returned instead.
+
+Dies if code can't be generated, or an error occured when compiling
+the code.
+
+If you just want to get the Perl code in a string, use C<emit_perl>.
+
 =head1 COMPARISON WITH OTHER DATA VALIDATION MODULES
 
 There are already a lot of data validation modules on CPAN. However, most of
@@ -738,50 +992,6 @@ are already several basic types each with a decent set of attributes. But some
 default values and filters. These will be added in future releases along with
 other planned features like variable substitution, etc.
 
-
-=head2 PERFORMANCE NOTES
-
-The way the code is written & structured (e.g. it uses Moose, validation
-involves a relatively high number of method calls, etc.) it is probably slower
-than other data validation modules. However, at the moment the code has not been
-profiled and optimized.
-
-To give a rough picture, here's how DS 0.03 fares on my Athlon 64 X2 5000+
-(which I think is still a fairly decent box in 2009). Perl 5.10.0, Moose 0.72.
-
-1. Using the simplest case:
-
- $validator->validate(1, "int")
-
-the speed is around 14,000 validations per second.
-
-2. Using the dice throws example (see DSM::Tutorial):
-
- $validator->validate([1,2,3,4,5,6,[1,1],[1,2],[1,3],[1,4]], $schema)
-
-the speed is around 150/sec.
-
-3. Using the dice throws example, but moving all subschemas to a hash and using
-DSP::LoadSchema::Hash to load it, the speed is around 190/sec.
-
-4. Using a fairly complex schema, XXX.
-
-With this kind of performance you might want to reconsider using DS inside
-functions that are called very frequently (like hundreds or thousands of times
-per second). But I think DS should be fine for CGI applications or for command
-line argument checking and I will not be focusing on performance for the time
-being.
-
-Some tips on performance:
-
-1. move subschemas out;
-
-2. keep schema simple;
-
-3. write heavy-duty validation logic in Perl (e.g. using new type handler and/or
-type attribute).
-
-
 =head1 SEE ALSO
 
 L<Data::Schema::Manual::Tutorial>,
@@ -792,10 +1002,6 @@ L<Data::Schema::Manual::Plugin>
 Some other data validation modules on CPAN: L<Data::FormValidator>, L<Data::Rx>,
 L<Kwalify>.
 
-=head1 AUTHOR
-
-Steven Haryanto, C<< <steven at masterweb.net> >>
-
 =head1 BUGS
 
 Please report any bugs or feature requests to C<bug-data-schema at
@@ -810,7 +1016,6 @@ You can find documentation for this module with the perldoc command.
 
     perldoc Data::Schema
 
-
 You can also look for information at:
 
 =over 4
@@ -833,20 +1038,18 @@ L<http://search.cpan.org/dist/Data-Schema/>
 
 =back
 
-
 =head1 ACKNOWLEDGEMENTS
 
+=head1 AUTHOR
 
-=head1 COPYRIGHT & LICENSE
+  Steven Haryanto <stevenharyanto@gmail.com>
 
-Copyright 2009 Steven Haryanto, all rights reserved.
+=head1 COPYRIGHT AND LICENSE
 
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This software is copyright (c) 2009 by Steven Haryanto.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
 
-__PACKAGE__->meta->make_immutable;
-no Moose;
-1;
