@@ -1,12 +1,39 @@
 package Data::Schema::Plugin::LoadSchema::Hash;
+our $VERSION = '0.12';
+
+
+# ABSTRACT: Plugin to load schemas from hashes
 
 use Moose;
 extends 'Data::Schema::Plugin::LoadSchema::Base';
 
+
+sub get_schema {
+    my ($self, $name) = @_;
+    my $found;
+
+    for my $h (@{ $self->validator->config->schema_search_path }) {
+        next unless ref($h) eq 'HASH' && exists $h->{$name};
+        return $h->{$name};
+    }
+    return;
+}
+
+__PACKAGE__->meta->make_immutable;
+no Moose;
+1;
+
+__END__
+=pod
+
 =head1 NAME
 
 Data::Schema::Plugin::LoadSchema::Hash - Plugin to load schemas from hashes
 
+=head1 VERSION
+
+version 0.12
+
 =head1 SYNOPSIS
 
     use Data::Schema;
@@ -27,33 +54,16 @@ Data::Schema::Plugin::LoadSchema::Hash - Plugin to load schemas from hashes
 Get schema from hashes, or C<undef> if not found. List of hashes to search from
 is specified in validator's C<schema_search_path> config variable.
 
-=cut
-
-sub get_schema {
-    my ($self, $name) = @_;
-    my $found;
-
-    for my $h (@{ $self->validator->config->schema_search_path }) {
-        next unless ref($h) eq 'HASH' && exists $h->{$name};
-        return $h->{$name};
-    }
-    return;
-}
-
 =head1 AUTHOR
 
-Steven Haryanto, C<< <steven at masterweb.net> >>
+  Steven Haryanto <stevenharyanto@gmail.com>
 
-=head1 COPYRIGHT & LICENSE
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009 Steven Haryanto, all rights reserved.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This software is copyright (c) 2009 by Steven Haryanto.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
 
-__PACKAGE__->meta->make_immutable;
-no Moose;
-1;
