@@ -1,18 +1,9 @@
 package Data::Schema::Type::Float;
+our $VERSION = '0.12';
 
-=head1 NAME
 
-Data::Schema::Type::Float - Type handler for floating point numbers ('float')
+# ABSTRACT: Type handler for floating point numbers ('float')
 
-=head1 SYNOPSIS
-
- use Data::Schema;
-
-=head1 DESCRIPTION
-
-This is the type handler for type 'float'.
-
-=cut
 
 use Moose;
 extends 'Data::Schema::Type::Num';
@@ -22,28 +13,52 @@ override handle_pre_check_attrs => sub {
     # XXX extra check when Num support other non-floats, e.g. complex, rational
 };
 
+sub short_english {
+    "float";
+}
+
 sub english {
     "float";
 }
 
+
+__PACKAGE__->meta->make_immutable;
+no Moose;
+1;
+
+__END__
+=pod
+
+=head1 NAME
+
+Data::Schema::Type::Float - Type handler for floating point numbers ('float')
+
+=head1 VERSION
+
+version 0.12
+
+=head1 SYNOPSIS
+
+ use Data::Schema;
+
+=head1 DESCRIPTION
+
+This is the type handler for type 'float'.
+
 =head1 TYPE ATTRIBUTES
 
 See L<Data::Schema::Type::Num>.
 
 =head1 AUTHOR
 
-Steven Haryanto, C<< <steven at masterweb.net> >>
+  Steven Haryanto <stevenharyanto@gmail.com>
 
-=head1 COPYRIGHT & LICENSE
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2009 Steven Haryanto, all rights reserved.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This software is copyright (c) 2009 by Steven Haryanto.
 
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
 
-__PACKAGE__->meta->make_immutable;
-no Moose;
-1;
